# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "au5800"


def test_decode_query(self):
    _query_message = "R 99123412W1234          123456"
    worklist = self.decode(_query_message)
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "123456"}],
    }
    assert worklist == expected


def test_decode_query_rerun(self):
    _query_message = "Rh02000101 0002        18940201"
    worklist = self.decode(_query_message)
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "18940201"}],
    }
    assert worklist == expected


def test_ignore_RB(self):
    _query_message = "RB00"
    worklist = self.decode(_query_message)
    expected = {"context": "query", "samples": [{"type": "patient", "sample_id": ""}]}
    assert worklist == expected


def test_ignore_RE(self):
    _query_message = "RE00"
    worklist = self.decode(_query_message)
    expected = {"context": "query", "samples": [{"type": "patient", "sample_id": ""}]}
    assert worklist == expected


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].specimen = "W"
    message = self.order(
        worklist=worklist,
        context={"query": self.parse("R 99123412W1234                    123456")},
    )
    expected = "S 123412W1234          123456    EF001  036666666           NYME ANO            A  0B  0"
    assert message == expected


def test_encode_order_no_birthdate(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].birthdate = None
    worklist.samples[0].specimen = "W"
    message = self.order(
        worklist=worklist,
        context={"query": self.parse("R 99123412W1234                    123456")},
    )
    expected = "S 123412W1234          123456    EF     036666666           NYME ANO            A  0B  0"
    assert message == expected


def test_encode_rerun_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].specimen = "W"
    message = self.order(
        worklist=worklist,
        context={"query": self.parse("Rh99123412W1234                    123456")},
    )
    expected = "Sh123412W1234          123456    EA  0B  0"
    assert message == expected


def test_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={"query": self.parse("R 99123412W1234                    123456")},
    )
    expected = "S 123412W1234                    E                                              "
    assert message == expected


def test_decode_qc(self):
    worklist = self.decode(
        "DQ00       Q001        IMMUNO-3 007E              110452625117126252544     3.39r       0492727184527271856     44.1r       0512280364322803653    260.0r       0522636146126361568     2.09r       0542549179325491797    18.06r       0562585731225857837     0.28r       0582574220525742250     3.21r       "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "IMMUNO-3",
                "results": [
                    {
                        "code": "045",
                        "date": None,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2625", "type": "reagent"},
                            {"lot": "1171", "type": "reagent"},
                            {"lot": "2625", "type": "reagent"},
                            {"lot": "2544", "type": "reagent"},
                        ],
                        "value": "3.39",
                    },
                    {
                        "code": "049",
                        "date": None,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2727", "type": "reagent"},
                            {"lot": "1845", "type": "reagent"},
                            {"lot": "2727", "type": "reagent"},
                            {"lot": "1856", "type": "reagent"},
                        ],
                        "value": "44.1",
                    },
                    {
                        "code": "051",
                        "date": None,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2280", "type": "reagent"},
                            {"lot": "3643", "type": "reagent"},
                            {"lot": "2280", "type": "reagent"},
                            {"lot": "3653", "type": "reagent"},
                        ],
                        "value": "260.0",
                    },
                    {
                        "code": "052",
                        "date": None,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2636", "type": "reagent"},
                            {"lot": "1461", "type": "reagent"},
                            {"lot": "2636", "type": "reagent"},
                            {"lot": "1568", "type": "reagent"},
                        ],
                        "value": "2.09",
                    },
                    {
                        "code": "054",
                        "date": None,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2549", "type": "reagent"},
                            {"lot": "1793", "type": "reagent"},
                            {"lot": "2549", "type": "reagent"},
                            {"lot": "1797", "type": "reagent"},
                        ],
                        "value": "18.06",
                    },
                    {
                        "code": "056",
                        "date": None,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2585", "type": "reagent"},
                            {"lot": "7312", "type": "reagent"},
                            {"lot": "2585", "type": "reagent"},
                            {"lot": "7837", "type": "reagent"},
                        ],
                        "value": "0.28",
                    },
                    {
                        "code": "058",
                        "date": None,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2574", "type": "reagent"},
                            {"lot": "2205", "type": "reagent"},
                            {"lot": "2574", "type": "reagent"},
                            {"lot": "2250", "type": "reagent"},
                        ],
                        "value": "3.21",
                    },
                ],
                "sample_id": "IMMUNO-3",
                "specimen": "S",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2022, 12, 29, 17, 52, 21)
    worklist = self.decode(
        "D 00000201 0001     33023610079    00                                             2022122917522110012601O0662601O451    53.89r       10032595476125956155    68.27r       00042668E4152668E863    35.03r       00062594085625940849   623.17r       00152640368026403682   177.86r       001626171583            37.01r       101926254958             9.98r       102225650547             2.26r       002326207460             5.39r       10252519780725198234         bar     10262592J3552592J034    11.20r       10272686193426862490     5.68r       10282399095223990947     1.93r       10302526426825264317     2.97r       10322580F1722580H002     1.29r       10342707D9402707E358     1.39r       00352655D4722655D879     3.11r       10452625117126252544     1.82r       10492727184527271856      7.6r       10512280364322803653    248.7r       00532548538825485314     1.83r       10542549179325491797    11.34r       00552549054825490565     0.83r       10582574220525742250     2.18r       0097                   142.64r       "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "0002_01",
                "results": [
                    {
                        "code": "001",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2601", "type": "reagent"},
                            {"lot": "O066", "type": "reagent"},
                            {"lot": "2601", "type": "reagent"},
                            {"lot": "O451", "type": "reagent"},
                        ],
                        "value": "53.89",
                    },
                    {
                        "code": "003",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2595", "type": "reagent"},
                            {"lot": "4761", "type": "reagent"},
                            {"lot": "2595", "type": "reagent"},
                            {"lot": "6155", "type": "reagent"},
                        ],
                        "value": "68.27",
                    },
                    {
                        "code": "004",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2668", "type": "reagent"},
                            {"lot": "E415", "type": "reagent"},
                            {"lot": "2668", "type": "reagent"},
                            {"lot": "E863", "type": "reagent"},
                        ],
                        "value": "35.03",
                    },
                    {
                        "code": "006",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2594", "type": "reagent"},
                            {"lot": "0856", "type": "reagent"},
                            {"lot": "2594", "type": "reagent"},
                            {"lot": "0849", "type": "reagent"},
                        ],
                        "value": "623.17",
                    },
                    {
                        "code": "015",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2640", "type": "reagent"},
                            {"lot": "3680", "type": "reagent"},
                            {"lot": "2640", "type": "reagent"},
                            {"lot": "3682", "type": "reagent"},
                        ],
                        "value": "177.86",
                    },
                    {
                        "code": "016",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "1583", "type": "reagent"},
                        ],
                        "value": "37.01",
                    },
                    {
                        "code": "019",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2625", "type": "reagent"},
                            {"lot": "4958", "type": "reagent"},
                        ],
                        "value": "9.98",
                    },
                    {
                        "code": "022",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2565", "type": "reagent"},
                            {"lot": "0547", "type": "reagent"},
                        ],
                        "value": "2.26",
                    },
                    {
                        "code": "023",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2620", "type": "reagent"},
                            {"lot": "7460", "type": "reagent"},
                        ],
                        "value": "5.39",
                    },
                    {
                        "code": "025",
                        "date": date,
                        "flags": ["b", "a", "r"],
                        "lots": [
                            {"lot": "2519", "type": "reagent"},
                            {"lot": "7807", "type": "reagent"},
                            {"lot": "2519", "type": "reagent"},
                            {"lot": "8234", "type": "reagent"},
                        ],
                        "value": "",
                    },
                    {
                        "code": "026",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2592", "type": "reagent"},
                            {"lot": "J355", "type": "reagent"},
                            {"lot": "2592", "type": "reagent"},
                            {"lot": "J034", "type": "reagent"},
                        ],
                        "value": "11.20",
                    },
                    {
                        "code": "027",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2686", "type": "reagent"},
                            {"lot": "1934", "type": "reagent"},
                            {"lot": "2686", "type": "reagent"},
                            {"lot": "2490", "type": "reagent"},
                        ],
                        "value": "5.68",
                    },
                    {
                        "code": "028",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2399", "type": "reagent"},
                            {"lot": "0952", "type": "reagent"},
                            {"lot": "2399", "type": "reagent"},
                            {"lot": "0947", "type": "reagent"},
                        ],
                        "value": "1.93",
                    },
                    {
                        "code": "030",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2526", "type": "reagent"},
                            {"lot": "4268", "type": "reagent"},
                            {"lot": "2526", "type": "reagent"},
                            {"lot": "4317", "type": "reagent"},
                        ],
                        "value": "2.97",
                    },
                    {
                        "code": "032",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2580", "type": "reagent"},
                            {"lot": "F172", "type": "reagent"},
                            {"lot": "2580", "type": "reagent"},
                            {"lot": "H002", "type": "reagent"},
                        ],
                        "value": "1.29",
                    },
                    {
                        "code": "034",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2707", "type": "reagent"},
                            {"lot": "D940", "type": "reagent"},
                            {"lot": "2707", "type": "reagent"},
                            {"lot": "E358", "type": "reagent"},
                        ],
                        "value": "1.39",
                    },
                    {
                        "code": "035",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2655", "type": "reagent"},
                            {"lot": "D472", "type": "reagent"},
                            {"lot": "2655", "type": "reagent"},
                            {"lot": "D879", "type": "reagent"},
                        ],
                        "value": "3.11",
                    },
                    {
                        "code": "045",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2625", "type": "reagent"},
                            {"lot": "1171", "type": "reagent"},
                            {"lot": "2625", "type": "reagent"},
                            {"lot": "2544", "type": "reagent"},
                        ],
                        "value": "1.82",
                    },
                    {
                        "code": "049",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2727", "type": "reagent"},
                            {"lot": "1845", "type": "reagent"},
                            {"lot": "2727", "type": "reagent"},
                            {"lot": "1856", "type": "reagent"},
                        ],
                        "value": "7.6",
                    },
                    {
                        "code": "051",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2280", "type": "reagent"},
                            {"lot": "3643", "type": "reagent"},
                            {"lot": "2280", "type": "reagent"},
                            {"lot": "3653", "type": "reagent"},
                        ],
                        "value": "248.7",
                    },
                    {
                        "code": "053",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2548", "type": "reagent"},
                            {"lot": "5388", "type": "reagent"},
                            {"lot": "2548", "type": "reagent"},
                            {"lot": "5314", "type": "reagent"},
                        ],
                        "value": "1.83",
                    },
                    {
                        "code": "054",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2549", "type": "reagent"},
                            {"lot": "1793", "type": "reagent"},
                            {"lot": "2549", "type": "reagent"},
                            {"lot": "1797", "type": "reagent"},
                        ],
                        "value": "11.34",
                    },
                    {
                        "code": "055",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2549", "type": "reagent"},
                            {"lot": "0548", "type": "reagent"},
                            {"lot": "2549", "type": "reagent"},
                            {"lot": "0565", "type": "reagent"},
                        ],
                        "value": "0.83",
                    },
                    {
                        "code": "058",
                        "date": date,
                        "flags": ["r"],
                        "lots": [
                            {"lot": "2574", "type": "reagent"},
                            {"lot": "2205", "type": "reagent"},
                            {"lot": "2574", "type": "reagent"},
                            {"lot": "2250", "type": "reagent"},
                        ],
                        "value": "2.18",
                    },
                    {
                        "code": "097",
                        "date": date,
                        "flags": ["r"],
                        "lots": [],
                        "value": "142.64",
                    },
                ],
                "sample_id": "33023610079",
                "specimen": "S",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_hil_result(self):
    date = datetime.datetime(2023, 1, 10, 10, 40, 16)
    worklist = self.decode(
        "D 00000201 0001          123456    EM061001049892             TEST AU1            202301101040160098                     0.24G L     00962608E808          0  0  0        "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "0002_01",
                "results": [
                    {
                        "code": "098",
                        "date": date,
                        "flags": ["G", "L"],
                        "lots": [],
                        "value": "0.24",
                    },
                    {
                        "code": "096",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {"lot": "2608", "type": "reagent"},
                            {"lot": "E808", "type": "reagent"},
                        ],
                        "value": "0  0  0",
                    },
                    {
                        "code": "L",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {"lot": "2608", "type": "reagent"},
                            {"lot": "E808", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "I",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {"lot": "2608", "type": "reagent"},
                            {"lot": "E808", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "H",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {"lot": "2608", "type": "reagent"},
                            {"lot": "E808", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                ],
                "sample_id": "123456",
                "specimen": "S",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
